@@ -5,10 +5,10 @@ NamedMatrix.py
 @author: Guy H. Fernald
 @created: 11/24/2009
 """
-
+import csv
+import sys
 import numpy
 import maxdot
-import csv
 from StringIO import StringIO
 
 H5PY_SUPPORT = True
@@ -104,18 +104,17 @@ class NamedMatrix(object):
         Override the [] method.
         """
         rows,cols = key
-        if type(rows) is list:
-            new_rownames = [self.rownames[i] for i in rows]
-        elif type(rows) is int:
+        if type(cols) is list or type(rows) is list:
+            return self._getitem_list(rows, cols)
+        
+        if type(rows) is int:
             new_rownames = [self.rownames[rows]]
-        else:
+        else: # slice
             new_rownames = self.rownames[rows]
         
-        if type(cols) is list:
-            new_colnames = [self.colnames[i] for i in cols]
-        elif type(cols) is int:
+        if type(cols) is int:
             new_colnames = [self.colnames[cols]]
-        else:
+        else: # slice
             new_colnames = self.colnames[cols]
         
         if type(cols) is int and type(rows) is int:
@@ -123,6 +122,33 @@ class NamedMatrix(object):
         
         return NamedMatrix(self._data.__getitem__(key), new_rownames, new_colnames)
     
+    def _getitem_list(self, rows, cols):
+        if type(cols) is slice:
+            start = cols.start if not cols.start is None else 0
+            stop = cols.stop if not cols.stop is None else len(self.colnames)
+            step = cols.step if not cols.step is None else 1
+            cols = range(start, stop, step)
+        elif type(cols) is int:
+            cols = [cols]
+        elif type(cols) is list:
+            cols = filter(lambda x: not x is None,[i if type(i) is int else self.column_index(i) for i in cols])
+        
+        if type(rows) is slice:
+            start = rows.start if not rows.start is None else 0
+            stop = rows.stop if not rows.stop is None else len(self.rownames)
+            step = rows.step if not rows.step is None else 1
+            rows = range(start, stop, step)
+        elif type(rows) is int:
+            rows = [rows]
+        elif type(cols) is list:
+            rows = filter(lambda x: not x is None, [i if type(i) is int else self.row_index(i) for i in rows])
+        
+        new_data = [[self._data[i,j] for j in cols] for i in rows]
+        new_rownames = [self.rownames[i] for i in rows]
+        new_colnames = [self.colnames[i] for i in cols]
+        
+        return NamedMatrix(new_data, new_rownames, new_colnames)
+    
     def __setitem__(self, key, value):
         """ Override the setter """
         if type(value) is NamedMatrix:
@@ -147,14 +173,16 @@ class NamedMatrix(object):
         Return the max-dot product of self and other.
         """
         if not self.colnames == other.rownames:
-            A = self.filter(columns=other.rownames)._data
-            B = other.filter(rows=self.colnames)._data
+            A = self[:, other.rownames]._data
+            B = other[self.colnames, :]._data
         else:
             A = self._data
             B = other._data
+        
         if not A.shape[1] == B.shape[0]:
             raise Exception("Cannot multiply matrices shapes do not match: %s, %s" % (A.shape, B.shape))
-        C = numpy.matrix(numpy.zeros((self.size(0), other.size(1))))
+        
+        C = numpy.matrix(numpy.zeros((A.shape[0], B.shape[1])))
         maxdot.maxdot(A, B, C)
         return NamedMatrix(C, self.rownames, other.colnames)
     
@@ -244,26 +272,16 @@ class NamedMatrix(object):
                 new[i,j] = fun(self[i,j])
         return new
     
-    # BUG: For some reason numpy.matrix doesn't like it when you give it two lists
-    # BUG: as indices. eg. matrix[[1,2],[0,3]]
-    # BUG: This bug shows it self here if you want to filter for both cols and rows.
-    def filter(self, rows = slice(None,None,None), columns = slice(None,None,None)):
+    # DEPRECATED: Filter has been deprecated. You can now directly index
+    # DEPRECATED: the NamedMatrix using row or columns names.
+    # DEPRECATED: (eg. A[:,[colname1, colname2, ...]])
+    def filter(self, rows = slice(None, None, None), columns = slice(None, None, None)):
         """
         Return a matrix with only the columns and rows that are given.
         """
-        if not type(rows) is slice:
-            rows = [self.row_index(r) for r in rows if not self.row_index(r) == None]
-            new_rownames = [self.rownames[i] for i in rows]
-        else:
-            new_rownames = self.rownames[rows]
-        
-        if not type(columns) is slice:
-            columns = [self.column_index(c) for c in columns if not self.column_index(c) == None]
-            new_colnames = [self.colnames[i] for i in columns]
-        else:
-            new_colnames = self.colnames[columns]
-        
-        return NamedMatrix(self._data[rows,columns], new_rownames, new_colnames)
+        print >> sys.stderr, "The filter method has been deprecated. You can now directly" + \
+            "index the NamedMatrix using row and column names. (eg. A[:,[colname1, colname2, ...]])"
+        return self[rows,columns]
     
     def save_loadable(self, file, label="NA"):
         """
